File type = "ooTextFile"
Object class = "TextGrid"

{{ textgrid.xmin }}
{{ textgrid.xmax }}
<{% if textgrid.tiers|length %}exists{% else %}absent{% endif %}> 
{{ textgrid.tiers|length }}
{% for tier in textgrid.tiers %}
"{{ "IntervalTier" if tier.interval_tier else "TextTier" }}"
"{{ tier.name }}"
{{ textgrid.xmin }}
{{ textgrid.xmax }}
{{ tier["items"]|length }}
{% for item in tier['items'] %}
{% if tier.interval_tier %}
{{ item.xmin }}
{{ item.xmax }}
"{{ item.text }}"
{% else %}
{{ item.number }}
"{{ item.mark }}"
{% endif %}
{% endfor %}
{% endfor %}
